﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AsyncExamples
{
    class Program
    {

        public static void Main(string[] args)
        {
            ///A way to call async methods in Main
            Task.Run(async () =>
            {
                Console.WriteLine("In Main before await");
                await MainAsync(args); //you need to create an asynchronous Main
                Console.WriteLine("In Main after await");
            }).GetAwaiter().GetResult();//and wait for it to finish
            Console.WriteLine("We can finish here");
            Console.ReadLine();
        }

        public static async Task MainAsync(string[] args) //when I have at least one await call, the method can be asynchronous
        //public static void Main(string[] args) //when we don't have await we have to do the old way
        {
            #region the usual synchronous method
            Console.WriteLine("\n--------------------the usual synchronous method-------------------");
            Worker worker1 = new Worker("Mietek");
            Console.WriteLine("We are delegating a synchronous task to worker");
            worker1.OrdinaryWork(10);
            Console.WriteLine("The synchronous task was delegated and done"); 
            Console.WriteLine("Press enter to go further");
            Console.ReadLine();
            #endregion

            #region Method creating a task
            Console.WriteLine("\n--------------------Method creating a task--------------------------");
            Worker worker2 = new Worker("Zenon");
            Console.WriteLine("We order to the employee work and he creates a Task");
            worker2.WorkTask(10);
            Console.WriteLine("Work for Zenon was commissioned and he began to perform. Unfortunately, here our supervision ends");
            Console.WriteLine("Press enter to go further");
            Console.ReadLine();
            #endregion

            #region method creating and returning task
            Console.WriteLine("\n--------------------method creating and returning task-------------");
            Worker worker3 = new Worker("Janusz");
            Console.WriteLine("We order to the employee work and he creates a Task");
            var januszTask = worker3.WorkAsync(10);
            Console.WriteLine("Work for Janusz was commissioned and he began to perform. Janusz returned the task so we can wait for it");
            januszTask.Wait();
            Console.WriteLine("Press enter to go further");
            Console.ReadLine();
            #endregion

            #region a method that returns a task and waits with await
            Console.WriteLine("\n--------------------a method that returns a task and waits with await-----");
            Worker worker4 = new Worker("Grażyna");
            Console.WriteLine("We commission the employee to work and he creates a Task");
            var grazynaTask = worker4.WorkAsync(10);
            Console.WriteLine("The work for Grażyna was commissioned and she began to perform.\nGrażyna returned task, so we can wait for it with await");
            await grazynaTask; //when we use the word await, the method we use await must be async.
            Console.WriteLine("Press enter to go further");
            Console.ReadLine();
            #endregion

            #region an example of a method using async await
            Console.WriteLine("\n--------------------an example of a method using async await-----------");
            Worker worker5 = new Worker("Stefan");
            Console.WriteLine("We are ordering the job to the worker Stefan and he's ordering the job farther");   
            var stefanTask = worker5.WorkOrderedAsync(10);
            Console.WriteLine("Work for Stefan was commissioned and he commissioned it further.\nStefan returned the task so we can wait for it with await");
            await stefanTask; //when we use the word await, the method we use await must be async.
            Console.WriteLine("Press enter to go further");
            Console.ReadLine();
            #endregion



        }
    }
}
