﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace CancelationExamples
{
    class Program
    {
        static void Main(string[] args)
        {
            ///Sposób na wywoływanie metod asynchronicznych w Mainie
            ///A way to call async methods in Main
            Task.Run(async () =>
            {
                Console.WriteLine("In Main before await");
                await MainAsync(args); //należy stworzyć sobie asynchronicznego Maina/you need to create an asynchronous Main
                Console.WriteLine("In Main after await");
            }).GetAwaiter().GetResult();//i poczekać na jego zakończenie/and wait for it to finish
            Console.WriteLine("We can end here");
            Console.ReadLine();
        }

        public static async Task MainAsync(string[] args) //gdy mam choć jedno zawołanie await to metoda może być asynchroniczna/when I have at least one await call, the method can be asynchronous
        //public static void Main(string[] args) //gdy nie mamy await musimy po staremu/when we don't have await we have to do the old way
        {
            #region Przykład z rzuceniem wyjątku OperationCancelException / Example with OperationCancelException throw
            Console.WriteLine("\n-------------------- method throwsing exception OCE -------------------");
            Progress<int> progressSimonTask = new Progress<int>(p => Console.WriteLine($"Worker Simon has already produced {p}"));
            CancellationTokenSource cts = new CancellationTokenSource(2000); //po 2 sekundach zarządaj przerwania/ request an interrupt after 2 seconds
            Worker worker1 = new Worker("Simon");
            Console.WriteLine("Order an employee to work asynchronously and have them report");
            Task simonTask = worker1.WorkAsync(10, progressSimonTask, cts.Token);

            try
            {
                await simonTask;
            }
            catch (OperationCanceledException oce)
            {
                Console.WriteLine("Simon's work was Canceled: " + oce.Message); //Praca Simona została przerwana
            }
            catch (InvalidOperationException ioe)
            {
                Console.WriteLine("During Simon's work we counght an exception: " + ioe.Message); //Podczas pracy Simona złapaliśmy wyjątek
            }
                
            Console.WriteLine($"Synchronous work ordered and completed with the status {simonTask.Status}"); //Praca synchroniczna zlecona i zakończona ze statusem {simonTask.Status} 
            Console.WriteLine("Press ENTER to continue"); //Aby kontynuować naciśnij ENTER 
            Console.ReadLine();
            #endregion

            #region Przykład ze zwykłym zakończeniem zadania / An example with the usual ending of a task
            Console.WriteLine("\n--------------------method ending when we have a cancellation request-------");//metoda kończąca się gdy mamy żądanie anulowania
            Progress<int> progressPiterTask = new Progress<int>(p => Console.WriteLine($"Worker Piter has already produced {p}")); //Pracownik Piotrek wypracował już {p}
            CancellationTokenSource cts2 = new CancellationTokenSource(2000); //po 2 sekundach zarządaj przerwania / request an interrupt after 2 seconds
            Worker worke2 = new Worker("Piter");
            Console.WriteLine("Order an employee to work asynchronously and have them report"); //Zlecamy pracownikowi pracę asynchroniczną i każemy raportować
            Task piterTask = worke2.Work2Async(10, progressPiterTask, cts2.Token);
            try
            {
                await piterTask;
            }
            catch (OperationCanceledException oce)
            {
                Console.WriteLine("Piter's work was Canceled: " + oce.Message); //Praca Piotra została przerwana:
            }
            catch (InvalidOperationException ioe)
            {
                Console.WriteLine("During Piters's work we counght an exception:: " + ioe.Message); //Podczas pracy piotra złapaliśmy wyjątek
            }

            Console.WriteLine($"Order an employee to work asynchronously and have them report {piterTask.Status}"); //Praca synchroniczna zlecona i zakończona ze statusem 
            Console.WriteLine("Press ENTER to continue");
            Console.ReadLine();
            #endregion

            #region canceling an entire group of tasks at once / anulowanie jednocześnie całej grupy zadań
            Console.WriteLine("\n-------------------- An example of ending a group of tasks-------"); //Przykład kończenia grupy zadań
            const int workersCount = 5;
            List<Progress<int>> progresses = new List<Progress<int>>(workersCount);
            List<Worker> brigade = new List<Worker>(workersCount);
            List<Task> tasks = new List<Task>(workersCount);
            CancellationTokenSource cts3 = new CancellationTokenSource(2000); //one common CTS. After 2 seconds, request an cancel/jeden wspólny CTS. Po 2 sekundach zarządaj przerwania

            for (int i = 0; i < workersCount; i++)
            {
                int a = i; //and what will happen if instead of 'a' we use straight 'i' in progress / a co będzie gdy zamiast a użyjemy wprost i w progress
              //  progresses.Add(new Progress<int>(p => Console.WriteLine($"Worker {i} has already produced {p}")));
                progresses.Add(new Progress<int>(p => Console.WriteLine($"Worker {a} has already produced {p}")));
                Worker worker = new Worker("Worker " + a.ToString());
                brigade.Add(worker);
                tasks.Add(worker.WorkAsync(10, progresses.Last(), cts3.Token));
             }
            try
            { 
              Task.WaitAll(tasks.ToArray()); //we are waiting for the completion of all tasks from the task table/czekamy na zakończenie wszystkich zadań z tablcy zadań. 
            }
            catch (OperationCanceledException oce)
            {
                Console.WriteLine("The work of the brigade has been cancelled: " + oce.Message); //Praca brygady została przerwana
            }
            catch (InvalidOperationException ioe)
            {
                Console.WriteLine("The work of the brigade has been cancelled: " + ioe.Message);
            }
            catch (AggregateException ae) //w przypadku wielu zadań dostaniemy wyjątek zagregowany. 
            {
                Console.WriteLine(ae.Message);
                foreach (var ex in ae.InnerExceptions)
                {
                    Console.WriteLine("I caught the exceptions: " + ex.Message);
                }
            }
            #endregion

            #region Inny przykład Task Cancelling /another Task Cancelling example
            Console.WriteLine("\n --------------Tasks Cancellation Example-------------");      // Przykład Anulowania Zadań
            //To interrupt the task, we have a CancellationToken class that monitors whether an interrupt request has been made for the object representing the task. 
            //The use of an interrupt token is the only recommended method for terminating a task.
            //The use of a token has 3 basic advantages: not everyone can interrupt the task (only the token holder), you can interrupt a group of tasks (just enter the same token), it improves the termination in the case of parallel loops and PLINQ

            //Do przerywania zadania mamy klasę CancellationToken monitorującą czy nastąpiło żądanie przerwania na rzecz obiektu reprezentującego zadanie. 
            //Użycie tokena przerwania jest jedyną zalecaną metodą przerywania zadania. 
            //Użycie tokena ma 3 podstawowe zalety: nie każdy może przerwać zadanie (tylko posiadacz tokena), można przerwać grupę zadań (wystarczy podać ten sam token), usprawnia przerywanie w przypadku pętli równoległych oraz PLINQ
            var tokenSource = new CancellationTokenSource();
            CancellationToken ct = tokenSource.Token; //before creating the task, we create a Token/przed utworzeniem zadania tworzymy Token

            Task task = Task.Factory.StartNew(() =>
            {
                Console.WriteLine("The task to cancel has started"); // Zadanie do anulowania wystartowało
 
                 ct.ThrowIfCancellationRequested(); //if the Cancel () method is called on tokenSource, an exception will be thrown here/jeżeli Na tokenSource wywołamy metodę Cancel() to zostanie tu rzucony wyjątek

                bool moreToDo = true;
                while (moreToDo)
                {
                    Console.WriteLine("The task to cancel is doing something there");//Zadanie do anulowania coś tam robi
                    Thread.Sleep(500);
                    if (ct.IsCancellationRequested) //you can also check for an interrupt request/można też sprawdzać czy nie nastpiło żądanie przerwania
                    {
                        //i obsłużyć takie przerwanie, np zapisując dane czy zamykając połączenia. 
                        //and handle such an interrupt, e.g. by saving data or closing connections.
                        ct.ThrowIfCancellationRequested();
                    }

                }
            }, tokenSource.Token); //token przekazujemy do obiektu zadania jako argument/we pass the token to the task object as an argument
            Thread.Sleep(2000);

            try
            {
                // task.Wait(2000);
                tokenSource.Cancel();
                task.Wait();
            }
            catch (OperationCanceledException oce)
            {
                Console.WriteLine("We have an exception{0}", oce.Message); //mamy wyjątek
            }
            catch (AggregateException e)
            {
                Console.WriteLine("We have an exception {0}", e.Message); //mamy wyjątek
                foreach (var item in e.InnerExceptions)
                {
                    Console.WriteLine("Error: {0}", item.Message);
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine("Caught a general exceptionk: {0}", ex.Message);//Złapany ogólny wyjątek   
            }
            finally
            {
                tokenSource.Dispose();
                Console.WriteLine("Finally the End");//Ostatecznie koniec
            }

            #endregion


        }
    }
}
