﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace CancelationExamples
{
    public class Worker
    {
        public string Name { get; } //name to recognize our worker
        public Worker(string name) //constructor forcing a name
        {
            Name = name;
        }


        public Task WorkAsync(int ile, IProgress<int> progres, CancellationToken ct)
        {
            Console.WriteLine($"Worker {Name} start working....");
            Task t = new Task(() =>
            {
              //  var _ct = ct;
          //      var _ct = (CancellationToken)o;
                for (int i = 1; i <= ile; i++)
                {
                    //Simpler approach / Prostrze podejście
                    if (ct != null) ct.ThrowIfCancellationRequested(); //if there is an interruption request, we throw the OperationCanceledException / jeżeli nastąpiło, żądanie przerwania, rzucamy wyjątkiem OperationCanceledException
                    //gdy rzucimy wyjątkiem wprost i nie podamy CancellaionToken to stan zadania nie będzie canceled tylko faulted, nie zostanie też odpalona kontynuacja OnlyOnCanceled 
                    //when we throw the exception directly and do not provide CancellationToken, the task status will not be canceled but faulted, and the OnlyOnCanceled continuation will not be fired
                    //if (_ct !=null && _ct.IsCancellationRequested) throw new OperationCanceledException($"Task of worker {Name} was cancelled");
                    // correct exception throwing. 
                    //  if (_ct != null && _ct.IsCancellationRequested) throw new OperationCanceledException($"Task of worker {Name} was canceled",_ct);
                    Thread.SpinWait(100000000); //simulate working
                    if (progres != null) progres.Report(i);
                }
                Console.WriteLine($"Worker {Name} finished task");
            },ct);
            //Thread.Sleep(3000); //zobaczmy co się stanie gdy zadanie nie będzie jeszcze wystartowane /let's see what happens when the task is not yet started
            Console.WriteLine($"The task status of worker {Name} before start is: {t.Status}");
            try
            {
                t.Start();
            }
            catch (InvalidOperationException ioe) //wykonanie start na zadaniu, które przeszło do stanu canceled powoduje rzucenie tego wyjątku /executing start on a job that has passed into the canceled state throws this exception
            {
                Console.WriteLine($"Task of {Name} was cancelled before start: " + ioe.Message);
            }
            Console.WriteLine($"The task status of worker {Name} after start is: {t.Status}");
            return t;
        }


        /// <summary>
        /// Druga wersja metody, ta nie rzuca wyjątku tylko elegancko się kończy. 
        /// </summary>
        /// <param name="ile"></param>
        /// <param name="progress"></param>
        /// <param name="ct"></param>
        /// <returns></returns>
        public Task Work2Async(int ile, IProgress<int> progress, CancellationToken ct)
        {
            Console.WriteLine($"Worker {Name} start working....");
            Task t = new Task(() =>
            {
                for (int i = 1; i <= ile; i++)
                {
                    if (ct != null && ct.IsCancellationRequested) //can end in a friendlier way/można zakończyć w bardziej przyjazny sposób
                    {
                        Console.WriteLine($"Worker {Name} regrets to stop work");
                        //  ct.ThrowIfCancellationRequested();
                        return;
                    }
                    Thread.SpinWait(100000000);
                    if (progress != null) progress.Report(i);
                }
                Console.WriteLine($"Worker {Name} finished task");
            }, ct);
            //Thread.Sleep(3000); //let's see what happens when the task is not yet started/zobaczmy co się stanie, gdy zadanie nie będzie jeszcze wystartowane 
            Console.WriteLine($"Task status of worker {Name} before start is: {t.Status}");
            try
            {
                t.Start();
            }
            catch (InvalidOperationException ioe) //executing start on a job that has passed into the canceled state throws this exception/wykonanie start na zadaniu które przeszło do stanu canceled powoduje rzucenie tego wyjątku
            {
                Console.WriteLine($"Task of {Name} was cancelled before start: " + ioe.Message); //praca pracownika X została przerwana przed startem
            }
            Console.WriteLine($"Task status of worker {Name} after start is: {t.Status}");
            return t;
        }
    }
}
