﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProgressExamples
{
    class Program
    {
        static void Main(string[] args)
        {
            ///Sposób na wywoływanie metod asynchronicznych w Mainie
            ///A way to call async methods in Main
            Task.Run(async () =>
            {
                Console.WriteLine("In Main before await / W Main przed await");
                await MainAsync(args); //należy stworzyć sobie asynchronicznego Maina/you need to create an asynchronous Main
                Console.WriteLine("In Main after await / W Main po await");
            }).GetAwaiter().GetResult();//i poczekać na jego zakończenie/and wait for it to finish
            Console.WriteLine("We can finish here / Tu możemy już kończyć");
            Console.ReadLine();
        }

        public static async Task MainAsync(string[] args) //gdy mam choć jedno zawołanie await to metoda może być asynchroniczna /when I have at least one await call, the method can be asynchronous
        //public static void Main(string[] args) //gdy nie mamy await musimy po staremu /when we don't have await we have to do the old way
        {
            #region Raportujemy postęp prac / We report on the progress of work
            Console.WriteLine("\n--------------------metoda z raportowaniem postępu prac-------------------");
            Console.WriteLine("\n--------------------method with progress reporting-------------------");
            Progress<int> anatolsProgress = new Progress<int>(p => Console.WriteLine($"Worker Anatol has produced {p}")); //za każdym razem gdy progress zaraportuje postęp, będzie wołany delegat piszący linię na ekran/each time progress reports progress, a delegate will be called to write a line to the screen
            Worker worker1 = new Worker("Anatol");
            Console.WriteLine("We order the employee to do asynchronous work and have them report");
            await worker1.PracaAsync(10,anatolsProgress);
            Console.WriteLine("Aynchronous work commissioned and performed");
            Console.WriteLine("Press Enter to continue...");
            Console.ReadLine();
            #endregion

            #region Raportujemy postęp prac za pomocą eventa / We report on the progress of work by event
            Console.WriteLine("\n--------------------metoda z raportowaniem postępu prac z eventem-------------");
            Console.WriteLine("\n--------method with reporting the progress of work with the event-------------");
            Progress<int> christopherProgress = new Progress<int>();
            christopherProgress.ProgressChanged += ChristopherProgress_ProgressChanged; //attaching a method to handle the event/podpięcie metody osbługującej eventa 
            Worker worker2 = new Worker("Christopher");
            Console.WriteLine("We order the employee to do asynchronous work and have them report");
            await worker2.PracaAsync(10, christopherProgress);
            Console.WriteLine("Aynchronous work commissioned and performed");
            Console.WriteLine("Press Enter to continue...");
            Console.ReadLine();
            #endregion

            #region Raportujemy postęp prac zwracając string / We report on the progress of work returning string
            Console.WriteLine("\n--------------------metoda z raportowaniem postępu prac zwracającym string-----");
            Console.WriteLine("\n---------------method with reporting the progress of work returning string-----");
            Progress<string> marryProgress = new Progress<string>(o => Console.WriteLine($"Marry is reporting: " + (string)o));
            Worker worker3 = new Worker("Marry");
            Console.WriteLine("We order the employee to do asynchronous work and have them report in a full sentence"); //Zlecamy pracownikowi pracę asynchroniczną i każemy raportować pełnym zdaniem
            await worker3.PracaAsync(10, marryProgress);
            Console.WriteLine("Aynchronous work commissioned and performed");
            Console.WriteLine("Press Enter to continue...");
            Console.ReadLine();
            #endregion
        }

        private static void ChristopherProgress_ProgressChanged(object sender, int e)
        {
            Console.WriteLine($"Worker Christopher has produced {e}");
        }
    }
}
