﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace TAP
{
    public class Piekarnik
    {
        public Piekarnik(string name)
        {
            Name = name;
        }

        public string Name { get; }

        //Synchroniczna metoda blokująca wywołującego do czasu zakończenia jej
        public void Rozgrzej(int temp)
        {
            Console.WriteLine("Piekarnik 0 włączony");
            for (int i = 0; i < temp; i = i + 10)
            {
                Thread.SpinWait(100000000);
                Console.WriteLine($"Synchroniczny 0 Rozgrzał do {i}");
            }
            Console.WriteLine("Piekarnik 0 rozgrzany");
        }

        //Metoda tworząca zadanie po wystartowaniu zadania można wrócić do wykonywania innych czynności
        public void RozgrzejTask(int temp)
        {
            Console.WriteLine("Piekarnik 1 włączony");
            Task t = new Task(() =>
            {
                for (int i = 0; i < temp; i = i + 10)
                {
                    Thread.SpinWait(100000000);
                    //await Task.Delay(TimeSpan.FromMilliseconds(100));
                    Console.WriteLine($"Task 1 Rozgrzał do {i}");
                }
                Console.WriteLine("Piekarnik1 rozgrzany");
            });
            t.Start();
        }

        //gdy zwrócimy wystartowane zadanie to taka metoda jest już tzw. awaitable
        public Task RozgrzejAsync(int temp)
        {
            Console.WriteLine("Piekarnik 2 włączony");
            Task t = new Task(() =>
            {
                for (int i = 0; i < temp; i = i + 10)
                {
                    Thread.SpinWait(100000000);
                    Console.WriteLine($"Async 2 Rozgrzał do {i}");
                }
                Console.WriteLine("Piekarnik2  rozgrzany");
            });
            t.Start();
            return t;
        }

        //Metodę awaitable można użyć w metodach asynchronicznych bez potrzeby tworzenia nowych zadań. Niestety metoda nic nie zwraca. 
        //Jest metodą typu uruchom i zapomnij
        public async void RozgrzejVoidMoreAsync(int temp)
        {
            Console.WriteLine("Piekarnik 2.1 włączony");
            await RozgrzejAsync(temp);
            Console.WriteLine("Piekarnik 2.1 metoda się kończy");
        }

        //ta metoda używa await i ma słowo async, też jest awaitable, jak widać nie ma tu tworzenia Tasków, nawet nie ma return; 
        //Zwrot potrzebnego zadania dodaje kompilator
        public async Task RozgrzejTaskMoreAsync(int temp)
        {
            Console.WriteLine("Piekarnik 2.2 włączony");
            await RozgrzejAsync(temp);
            Console.WriteLine("Piekarnik 2.2 metoda się kończy");
        }

        //możemy dodać raportowanie postępu z każdym zwiększeniem temeratury progress bedzie raportował nową temp
        public Task RozgrzejTaskAsync(int temp, IProgress<int> progress)
        {
            Task t = new Task(() =>
            {
                for (int i = 0; i < temp; i = i + 10)
                {
                    Thread.SpinWait(100000000);
                    Console.WriteLine($"Async 3 Rozgrzał do {i}");
                    progress.Report(i);
                }
                Console.WriteLine("Piekarnik 3 rozgrzany");
            });
            t.Start();
            return t;
        }

        public Task RozgrzejTaskAsync4(int temp, IProgress<int> progress, CancellationToken cancellationToken)
        {

            Task t = new Task(() =>
            {
                for (int i = 0; i < temp; i = i + 10)
                {
                    cancellationToken.ThrowIfCancellationRequested(); //można rzucić wyjątek
                    //if (cancellationToken.IsCancellationRequested) //albo obsłużyć w bardziej przyjazny sposób
                    //{
                    //    Console.WriteLine("Wyłączam piekarnik 4");
                    //    return;
                    //}
                    Thread.SpinWait(100000000);
                    Console.WriteLine($"Async 4 Rozgrzał do {i}");
                    progress.Report(i);
                }
                Console.WriteLine("Piekarnik 4 rozgrzany");
            }, cancellationToken);
            t.Start();
            return t;
        }

        /// <summary>
        /// Funkcja zwraca przez zadanie wartość bool, czy udało się rozgrzać czy wyłączyliśmy przed czasem. 
        /// </summary>
        /// <param name="temp">Zadana temeratura</param>
        /// <param name="name">Nazwa naszego piekarnika</param>
        /// <param name="progress"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        public Task<bool> RozgrzejTaskAsync(int temp, IProgress<int> progress, CancellationToken cancellationToken)
        {

            Task<bool> t = new Task<bool>(() =>
            {
                for (int i = 0; i < temp; i = i + 10)
                {
                    //cancellationToken.ThrowIfCancellationRequested(); //można rzucić wyjątek
                    if (cancellationToken.IsCancellationRequested) //albo obsłużyć w bardziej przyjazny sposób
                    {
                        Console.WriteLine($"Wyłączam piekarnik {Name}");
                        return false;
                    }

                    Thread.SpinWait(100000000);
                    Console.WriteLine($"Async {Name} Rozgrzał do {i}");
                    progress.Report(i);
                }
                Console.WriteLine($"Piekarnik {Name} rozgrzany");
                return true;
            }, cancellationToken);
            t.Start();
            return t;
        }


        
         

        public async Task<bool> PieczTaskAsync(int ile, IProgress<int> progress, CancellationToken cancellationToken)
        {
            CancellationTokenSource cts;
            
            for (int i = 0; i < ile; i++)
            {
                using (cts = new CancellationTokenSource()) //konstrukcja używana dla obiektów disposable (pod koniec bloku using obiekt jest usuwany, w celu utworzenia nowego.

                {
                   
                    var heavyTask = LoadCreator.MakeSomeLoadAsync(cts.Token);

                    if (cancellationToken.IsCancellationRequested)
                    {
                        Console.WriteLine($"Wyłączam piekarnik {Name}");

                        cts.Cancel(); //zakończ proces obciążający
                        return false;
                    }

                    try
                    {
                        cts.CancelAfter(1000); //po 1 sekundzie zakończ proces obciążający
                        await heavyTask;
                    }
                    catch (OperationCanceledException)
                    {
                        Console.WriteLine("heavyTask status: {0}",heavyTask.Status);
                    }
                }
                progress.Report(i + 1);
            }
            return true;
        }

        //public async Task<bool> RozgrzejAsync(int temp, string name, IProgress<int> progress, CancellationToken cancellationToken)
        //{
        //    return await Task<bool>.Run(() =>
        //    {
        //        for (int i = 0; i < temp; i++)
        //        {
        //            if (cancellationToken.IsCancellationRequested)
        //            {
        //                Console.WriteLine("Wyłączam piekarnik");
        //                return false;
        //            }
        //            //await 
        //            Task.Delay(TimeSpan.FromMilliseconds(100), cancellationToken);
        //            progress.Report(i);
        //        }
        //        Console.WriteLine($"Piekarnik {name} rozgrzany");
        //        return true;
        //    });
        //}
    }
}
